// *************************************************************************
// [内容] 掲示板介入 content_scripts スクリプト。
// [詳細] 掲示板への各種施術を実現するためのメインスクリプト。
// [更新] 2023/04/30 00:00
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

// [定義] コメント内容が挿入されるエレメントID。
var esViewComment = "view_comment";

// [施術有効フラグ] フィルタリング実施対象のフラグ。
var fRemove = false;
var fMarkup = false;
var fPerfectly = false;
var fBadNicknameRemove = false;
var fStalkingRemove = false;
var fSpamRemove = true;
var SpamLevel = 3; // このレベル以上で判定されたスパムを除去する

// [配列] フィルタリング設定を保持する配列。(正規表現ニックネーム)
var aRemoveRegularUsername = [];
var aMarkupRegularUsername = [];

// [配列] フィルタリング設定を保持する配列。(完全一致ニックネーム)
var aRemoveUsername = [];
var aMarkupUsername = [];

// [配列] フィルタリング設定を保持する配列。(ユーザーID)
var aRemoveUserid = [];
var aMarkupUserid = [];

// [フラグ] ユーザーIDストレージの更新を許可するときはtrue
var fUserIDStorageRefresh = true;

// [配列] ログイン状態
var login = {
    badname: false
};

// [イベント受信関数] アンロード
function onBoardUnload() {
    browser.storage.onChanged.removeListener(onBoardStorageChanged);
}

// [イベント受信関数] ストレージ変化時の動的介入
function onBoardStorageChanged(changes, namespace) {
    var fUserFiltering = false;
    if (namespace == "local") {
        if (changes.kjclub_userfiltering) {
            fUserFiltering = true;
            fRemove = changes.kjclub_userfiltering.newValue;
        }
        if (changes.kjclub_userfiltering_perfectly) {
            fUserFiltering = true;
            fPerfectly = changes.kjclub_userfiltering_perfectly.newValue;
        }
        if (changes.kjclub_userfiltering_badnickname) {
            fUserFiltering = true;
            fBadNicknameRemove = changes.kjclub_userfiltering_badnickname.newValue;
        }
        if (changes.kjclub_userfiltering_blacklist_stalking) {
            fUserFiltering = true;
            fStalkingRemove = changes.kjclub_userfiltering_blacklist_stalking.newValue;
        }
        if (changes.kjclub_userfiltering_blacklist_spam) {
            fUserFiltering = true;
            fSpamRemove = changes.kjclub_userfiltering_blacklist_spam.newValue;
        }
        if (changes.kjclub_userfiltering_blacklist_spam_level) {
            fUserFiltering = true;
            SpamLevel = changes.kjclub_userfiltering_blacklist_spam_level.newValue;
        }

        if (changes.kjclub_usermarking) {
            fUserFiltering = true;
            fMarkup = changes.kjclub_usermarking.newValue;
        }
        if (changes.kjclub_userfiltering_nicknames) {
            fUserFiltering = true;
            aRemoveRegularUsername = SplitListRegularNickname(changes.kjclub_userfiltering_nicknames.newValue);
            aRemoveUsername = SplitListNickname(changes.kjclub_userfiltering_nicknames.newValue);
            Array.prototype.push.apply(aRemoveUsername, AryDecodeNum(aRemoveUsername));
            aRemoveUserid = SplitListUserid(changes.kjclub_userfiltering_nicknames.newValue);
        }
        if (changes.kjclub_usermarking_nicknames) {
            fUserFiltering = true;
            aMarkupRegularUsername = SplitListRegularNickname(changes.kjclub_usermarking_nicknames.newValue);
            aMarkupUsername = SplitListNickname(changes.kjclub_usermarking_nicknames.newValue);
            Array.prototype.push.apply(aMarkupUsername, AryDecodeNum(aMarkupUsername));
            aMarkupUserid = SplitListUserid(changes.kjclub_usermarking_nicknames.newValue);
        }
        if (changes.kjclub_currentlogin_badname) {
            login.badname = changes.kjclub_currentlogin_badname.newValue;
        }
    }

    // スレッド内容送信時の介入を開始
    CheckInWriteSubmit();

    if (fUserFiltering) {
        // ストレージ変化時の再更新時は、ユーザーIDストレージの更新を禁止する
        fUserIDStorageRefresh = false;

        // スレッド一覧への介入
        ThreadsAnalyze();

        // コメント一覧への介入
        CommentsAnalyze();

        // スレッド書き込みフォーム・ボタンへの介入
        ThreadPostRebuild();

        // ユーザーIDストレージの更新を許可する
        fUserIDStorageRefresh = true;
    }
}

// [関数] Ajaxで挿入されるコメントへの介入のため、DOM監視を開始する関数。
// [補足] タイミングによっては監視前に挿入が実行される場合があるため、介入関数を最初に一度実行する。
function InitializeCommentChecker() {
    // 対象とするノードを取得
    var target = document.getElementById(esViewComment);

    if (!target) {
        return;
    }

    // オブザーバインスタンスを作成
    const observer = new MutationObserver((mutations) => {
        mutations.forEach((mutation) => {
            if (mutation.type == "childList") {

                // console.log("childList Observer");
                // observer.disconnect();

                // 内容に介入する(2回目)
                CommentsAnalyze();
            }
        });
    });

    // 対象ノードとオブザーバの設定を渡す
    observer.observe(target, {
        attributes: false,
        childList: true,
        characterData: false
    });

    // 内容に介入する(1回目)
    CommentsAnalyze();
}

// [関数] コメント一覧の介入を実施する。
function CommentsAnalyze() {
    var elThreadsTable = null;
    var records = 0;
    var stripe = false;

    // console.log("KJCLUB Plus : CommentsAnalyze");

    try {
        // URLのパスで情報確認
        var curInfo = GetCurrentInformation();
        if (curInfo == null) {
            // 掲示板ではないので戻る
            console.log("掲示板ではない");
            return;
        }

        // 対象とするノードを取得
        var target = document.getElementById(esViewComment);

        // コメント一覧エリアが存在しない場合は介入しない
        if (!target) {
            // console.log("KJCLUB Plus : CommentsAnalyze : コメント一覧エリアが存在しない");
            return;
        }

        // この文書はスレッド内容であるとみなし、スレッド本文を記述したオーナーの情報を収得する
        var ownerid = null;
        var ownername = null;

        var elPanelBodys = document.getElementsByClassName("panel-body");
        if (elPanelBodys.length >= 1) {
            var elOwnerHeader = elPanelBodys[0];
            // ノード内のaタグのリストを得る
            var eDs = elOwnerHeader.getElementsByTagName('a');
            if (eDs.length >= 1) {
                var elOwnerID = eDs[0];
                var s = elOwnerID.getAttribute("onclick");
                if (s) {
                    // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'/) || [])[1]; // ～2023/3/28までのKJに対応
                    // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'\s*,\s*\'(\w*)\'/) || [])[2]; // 2023/3/30～2023/4/19のKJ改訂対応
                    var result = (s.match(/^javascript:showSideView\(this,\s*\'(.*)\'\s*,\s*(\")/) || [])[2]; // 2023/4/19 11:58:14のKJ改訂対応
                    if (result != undefined) {
                        // オーナーのニックネームを検索
                        var elSs = elOwnerID.getElementsByTagName("strong");
                        if (elSs.length >= 1) {
                            // ownerid = result;
                            // if (ownerid != null) {
                            // ownername2 = result;
                            // if (username2 != null) {
                            // ownername = elSs[0].textContent.trim();
                            ownername = elSs[0].textContent; // トリムしない
                            // }
                        }

                        // コメントレコードのヘッダにIDを表示させる拡張
                        AddUserIDContentOnComment(elOwnerID, ownerid, ownername, "このスレッドの作成者です");
                    }
                }
            }
        }

        // ノード内のaタグのリストを得る
        var eDs = target.getElementsByTagName('a');

        // for (var eDi = 0; eDi < eDs.length; eDi++) {
        for (var eDi = eDs.length - 1; eDi >= 0; eDi--) {
            var el = eDs[eDi]; // プロファイルオープナーのリンクタグ
            var s = el.getAttribute("onclick");

            if (s) {
                // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'/) || [])[1]; // ～2023/2/28までのKJに対応
                // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'\s*,\s*\'(\w*)\'/) || [])[2]; // 2023/3/30～2023/4/19のKJ改訂対応
                var result = (s.match(/^javascript:showSideView\(this,\s*\'(.*)\'\s*,\s*(\")/) || [])[2]; // 2023/4/19 11:58:14のKJ改訂対応
                if (result != undefined) {
                    if (records == 0) {
                        elThreadsTable = el.parentNode.parentNode.parentNode.parentNode;
                    }

                    records++;

                    var message = "";

                    var userid = null;
                    var username = null;

                    var elSs = el.getElementsByTagName("strong");
                    if (elSs.length >= 1) {
                        // userid = result;
                        // if (userid != null) {
                        // username2 = result;
                        // if (username2 != null) {
                        // username = elSs[0].textContent.trim();
                        username = elSs[0].textContent; // トリムしない
                        // }
                    }

                    // if (userid == null || username == null) {
                    if (username == null) {
                        continue;
                    }

                    el.setAttribute('cell', 'user');

                    // 日付・時刻の要素を調べる
                    var dDate = new Date();

                    try {
                        // デスクトップ版とスマートフォン版で共通の解析
                        var sDate = el.nextElementSibling.textContent.trim();
                        // console.log( "date="+sDate );
                        dDate = BuildDate(sDate);
                        // console.log( dDate.toString() );
                    } catch (e) {
                        // console.log(e);
                    }

                    // レコード位置ベース
                    var elRecordBase = el.parentNode.parentNode;

                    // el.setAttribute("ondblclick","textSelect(this)");

                    // レコード位置
                    var elRecord = elRecordBase.parentNode.parentNode;

                    // 国判定
                    var nation = "ZZ";
                    var elSpans = elRecord.getElementsByTagName("span");
                    if (elSpans) {
                        for (var i = 0; i < elSpans.length; i++) {
                            var span = elSpans[i];
                            var cls = span.getAttribute("class");
                            if (cls) {
                                if (cls == "flag-icon flag-icon-jp") {
                                    nation = "JP";
                                    break;
                                } else
                                if (cls == "flag-icon flag-icon-kr") {
                                    nation = "KR";
                                    break;
                                }
                            }
                        }
                    }

                    // レコードに付加情報
                    elRecord.className = "panel-record";

                    elRecord.setAttribute('type', 'list_record');
                    elRecord.setAttribute('kjuserid', userid);
                    elRecord.setAttribute('nation', nation);
                    elRecord.setAttribute('stripe', stripe);

                    // ID情報のストレージ
                    StorageIDInformation({
                        userid: userid,
                        username: username,
                        nation: nation,
                        category: curInfo.category,
                        datetime: dDate.getTime()
                    });

                    var found = 0;
                    var fPerfectRemoved = false;
                    var sDisplay = "show";

                    // var faker = checkFaker(userid, username, ownerid);
                    // var badname = isBadNickname(username);

                    if (fMarkup) {
                        found = checkMark(userid, username);
                        if (found) {
                            message = "強調 (" + ChkMessageById(found) + ")";
                            sDisplay = 'markup';
                        }
                    }

                    if (!found) {
                        found = checkRemove(userid, username, ownerid, elRecord, curInfo);
                        if (fRemove) {
                            if (found) {
                                elRecord.setAttribute('data-remove', true);
                                message = "除去 (" + ChkMessageById(found) + ")";
                                if (fPerfectly) {
                                    sDisplay = 'hidden';
                                    fPerfectRemoved = true;
                                } else {
                                    sDisplay = 'opacity';
                                }
                            }
                        } else {
                            if (found) {
                                message = "注意 (" + ChkMessageById(found) + ")";
                            }
                        }
                    }

                    /*
					if (fRemove && fBadNicknameRemove && isBadNickname(username)) {
						if (message != "") {
							message = message + "\n";
						}
						message = message + "なりすまし等の疑いがある不穏な加工がされたニックネームです";
						if (!found) {
							// 除去・強調が特別になされていないので、特殊な除去処理を実施する
							found = true;
							if (message != "") {
								message = message + "\n";
							}
							message = message + "除去 (不穏加工ニックネーム判定)";
							if (fPerfectly) {
								sDisplay = 'hidden';
								fPerfectRemoved = true;
							} else {
								sDisplay = 'opacity';
							}
						}
					}
					*/

                    elRecord.setAttribute('display', sDisplay);

                    // コメントレコードのヘッダにIDを表示させる拡張 (2017.8.6.59)
                    AddUserIDContentOnComment(el, userid, username, message);

                    elRecord.setAttribute('record', records);

                    stripe = !stripe;
                }
            }
        }

        // ログインユーザーが不穏なニックネームを使用している場合の介入処理
        CommentPostRebuild();
    } catch (e) {
        // DOM解析中エラー時はチェックを抜ける。
        // console.log("KJCLUB Plus : KJAnalyze : board.js : CommentsAnalyze : error : " + e)
    }
    // console.log("KJCLUB Plus : KJAnalyze : board.js : CommentsAnalyze : "+records+" comment records found.")
}

// [関数] スレッド一覧の介入を実施する。
function ThreadsAnalyze() {
    var elThreadsTable = null;
    var records = 0;
    var stripe = false;

    try {
        // URLのパスで情報確認
        var curInfo = GetCurrentInformation();
        if (curInfo == null) {
            // 掲示板ではないので戻る
            // console.log("掲示板ではない");
            return;
        }

        var eDs = document.getElementsByTagName('a');
        var eAHead = 0;
        for (var eDi = eDs.length; eDi--;) {
            var el = eDs[eDi]; // プロファイルオープナーのリンクタグ。
            var s = el.getAttribute("href");
            if (s) {
                // console.log('href='+s)
                if (s.match(/^\?&sst=/) || s.match(/^\?sst=/)) {
                    eAHead = eDi;
                    break;
                }
            }
        }

        // スレッドリスト
        // for (var eDi = eAHead; eDi < eDs.length; eDi++) {
        for (var eDi = eDs.length - 1; eDi >= eAHead; eDi--) {
            var el = eDs[eDi]; // プロファイルオープナーのリンクタグ
            var s = el.getAttribute("onclick");

            if (s) {
                // javascript:showSideView(this, '','curryudon', &quot;currypakupaku&quot;, '', 'board');
                // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'/) || [])[1]; // ～2023/3/28までのKJ
                // var result = (s.match(/^javascript:showSideView\(this,\s*\'(\w*)\'\s*,\s*\'(\w*)\'/) || [])[2]; // 2023/3/30のKJ改訂対応
                var result = (s.match(/^javascript:showSideView\(this,\s*\'(.*)\'\s*,\s*(\")/) || [])[2]; // 2023/4/19 11:58:14のKJ改訂対応
                if (result != undefined) {
                    if (records == 0) {
                        elThreadsTable = el.parentNode.parentNode.parentNode.parentNode;
                    }
                    records++;

                    var message = "";

                    var userid = null;
                    var username = null;

                    var elSs = el.getElementsByTagName("strong");
                    if (elSs.length >= 1) {
                        // userid = result;
                        // if (userid != null) {
                        username = elSs[0].textContent; // .trim();
                        // }
                    }

                    // if (userid == null || username == null) {
                    if (username == null) {
                        continue;
                    }

                    // 日付・時刻の要素を調べる
                    var dDate = new Date();

                    try {
                        if (isMobileDocument()) {
                            // スマートフォン版ブラウザの場合
                            var sDate = el.parentNode.textContent.trim();
                            sDate = sDate.substr(sDate.lastIndexOf("(") + 1);
                            // console.log( "date(mobile)="+sDate );
                            dDate = BuildDate(sDate.replace(")", "").trim());
                            // console.log( dDate.toString() );
                        } else {
                            // デスクトップ版ブラウザの場合
                            var elDate = el.parentNode.nextElementSibling;
                            var elHit = elDate.nextElementSibling;
                            var elRecommend = elHit.nextElementSibling;
                            // console.log( "date="+elDate.textContent );
                            // console.log( "hit="+elHit.textContent );
                            // console.log( "recommend="+elRecommend.textContent );
                            dDate = BuildDate(elDate.textContent.trim());
                            // console.log( dDate.toString() );
                        }
                    } catch (e) {
                        // console.log(e);
                    }

                    el.setAttribute('cell', 'user');

                    // レコード位置
                    var elRecord = el.parentNode.parentNode;

                    // 国判定
                    var nation = "ZZ";
                    var elSpans = elRecord.getElementsByTagName("span");
                    if (elSpans) {
                        for (var i = 0; i < elSpans.length; i++) {
                            var span = elSpans[i];
                            var cls = span.getAttribute("class");
                            if (cls) {
                                if (cls == "flag-icon flag-icon-jp") {
                                    nation = "JP";
                                    break;
                                } else
                                if (cls == "flag-icon flag-icon-kr") {
                                    nation = "KR";
                                    break;
                                }
                            }
                        }
                    }

                    // レコード情報を付加
                    elRecord.setAttribute('type', 'list_record');
                    elRecord.setAttribute('kjuserid', userid);
                    elRecord.setAttribute('nation', nation);
                    elRecord.setAttribute('stripe', stripe);

                    // ID情報のストレージ
                    StorageIDInformation({
                        userid: userid,
                        username: username,
                        nation: nation,
                        category: curInfo.category,
                        datetime: dDate.getTime()
                    });

                    // 除去・強調の処理
                    var found = 0;
                    var fPerfectRemoved = false;
                    var sDisplay = "show";

                    if (fMarkup) {
                        found = checkMark(userid, username);
                        if (found) {
                            message = "強調 (" + ChkMessageById(found) + ")";
                            sDisplay = 'markup';
                        }
                    }

                    if (!found) {
                        found = checkRemove(userid, username, null, elRecord, curInfo);
                        if (fRemove) {
                            if (found) {
                                elRecord.setAttribute('data-remove', true);
                                message = "除去 (" + ChkMessageById(found) + ")";
                                if (fPerfectly) {
                                    sDisplay = 'hidden';
                                    fPerfectRemoved = true;
                                } else {
                                    sDisplay = 'opacity';
                                }
                            }
                        } else {
                            if (found) {
                                message = "注意 (" + ChkMessageById(found) + ")";
                            }
                        }
                    }

                    /*
					if (fRemove && fBadNicknameRemove && isBadNickname(username)) {
						if (message != "") {
							message = message + "\n";
						}
						message = message + "なりすまし等の疑いがある不穏な加工がされたニックネームです";
						if (!found) {
							// 除去・強調が特別になされていないので、特殊な除去処理を実施する
							found = true;
							if (message != "") {
								message = message + "\n";
							}
							message = message + "除去 (不穏加工ニックネーム判定)";
							if (fPerfectly) {
								sDisplay = 'hidden';
								fPerfectRemoved = true;
							} else {
								sDisplay = 'opacity';
							}
						}
					}
					*/

                    elRecord.setAttribute('display', sDisplay);

                    if (!fPerfectRemoved) {
                        if (el.getBoundingClientRect().width <= 4) {
                            // クリックしづらいように細工してあるニックネームの疑い
                            if (message != "") {
                                message = message + "\n";
                            }
                            message = message + i18n('kjclub_caution_user_nickname_hidden_hack', 'Hiden nickname hack');

                            var elSp = document.createElement('span');
                            el.style.color = "blue";
                            var elText = document.createTextNode('※');
                            elSp.appendChild(elText);

                            elSp.setAttribute("title", UserTooltipString(userid, username, message));

                            el.appendChild(elSp);
                        }
                    }

                    // スレッドレコードのヘッダにIDを表示させる拡張 (2017.8.6.59)
                    AddUserIDContentOnThread(el, userid, username, message);

                    elRecord.setAttribute('record', records);

                    stripe = !stripe;
                }
            }
        }
        if (elThreadsTable != null) {
            elThreadsTable.setAttribute('records', records);
        }
    } catch (e) {
        // DOM解析中エラー時はチェックを抜ける。
        // console.log("KJCLUB Plus : KJAnalyze : board.js : ThreadsAnalyze : error : "+e)
    }
}

// [関数] モバイル判定
function isMobileDocument() {
    return document.getElementById("pnProductNavContents") ? true : false;
}

// [関数] テキストから日時データーを作成
function BuildDate(sDate) {
    // 現在時刻から日付時刻を補完する
    var dNow = new Date();

    //年・月・日・曜日を取得する
    var year = dNow.getFullYear();
    var month = dNow.getMonth();
    var week = dNow.getDay();
    var day = dNow.getDate();
    var hours = dNow.getHours();
    var minutes = dNow.getMinutes();
    var seconds = dNow.getSeconds();

    var fmtHourMin = sDate.match(/^(\d+):(\d+)$/);
    if (fmtHourMin == null) {
        var fmtMonDay = sDate.match(/^(\d+)-(\d+)$/);
        if (fmtMonDay == null) {
            var fmtYearMonDay = sDate.match(/^(\d+)-(\d+)-(\d+)$/);
            if (fmtYearMonDay == null) {
                var fmtYearMonDayHourMin = sDate.match(/^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+)$/);
                if (fmtYearMonDayHourMin == null) {
                    // 無効な日付フォーマット
                    return dNow; // 現在の日付を返答
                }
                // 年-月-日 時:分
                year = Number(fmtYearMonDayHourMin[1]);
                if (year < 1900) {
                    year += 2000;
                }
                return new Date(year, Number(fmtYearMonDayHourMin[2]) - 1, Number(fmtYearMonDayHourMin[3]), Number(fmtYearMonDayHourMin[4]), Number(fmtYearMonDayHourMin[5]), 0);
            }
            // 年-月-日
            year = Number(fmtYearMonDay[1]);
            if (year < 1900) {
                year += 2000;
            }
            return new Date(year, Number(fmtYearMonDay[2]) - 1, Number(fmtYearMonDay[3]), 0, 0, 0);
        }
        // 月-日
        return new Date(year, Number(fmtMonDay[1]) - 1, Number(fmtMonDay[2]), 0, 0, 0);
    }
    // 時:分
    return new Date(year, month, day, Number(fmtHourMin[1]), Number(fmtHourMin[2]), 0);
}

// [関数] ドキュメントのURLからカテゴリー文字列(「exc_board_9」等)を抽出する
function GetCurrentInformation() {
    var ary = location.pathname.split("/"); // /jp/board/exc_board_9
    if (ary.length >= 4 && (ary[1] == "jp" || ary[1] == "kr") && ary[2] == "board") {
        var found = ary[3].match(/^(exc_board_\d+)$/);
        if (found != null) {
            return {
                nation: ary[1],
                service: ary[2],
                category: found[0]
            };
        }
    }
    return null;
}

// [関数] 一致原因コードをメッセージに変換
var aryFoundCodes = ["", "IDが一致", "ニックネームが完全一致", "正規表現でニックネームが一致", "ストーキングブラックリストに一致", "スパムブラックリストに一致", "フェイカーブラックリストに一致", "不穏加工ニックネーム判定", "不穏加工ニックネーム判定かつフェイカーブラックリストに一致"];

function ChkMessageById(foundCode) {
    if (aryFoundCodes.length <= foundCode) {
        return null;
    }
    return aryFoundCodes[foundCode];
}

// [関数] 除去対象か判定
function checkRemove(userid, username, ownerid, el, curInfo) {
    let badname = isBadNickname(username);
    let faker = checkFaker(userid, username, ownerid);
    let spam = checkSpam(userid, curInfo.category);
    if (el) {
        el.setAttribute('data-badname', badname);
        el.setAttribute('data-faker', faker);
        el.setAttribute('data-spam', spam);
        el.setAttribute('data-stalker', false);
    }
    if (!fRemove) {
        el.setAttribute('data-remove', false);
        if (badname && faker) {
            return 8;
        }
        if (badname) {
            return 7;
        }
        if (faker) {
            return 6;
        }
        if (spam) {
            return 5;
        }
        return 0;
    }
    // 正規表現ニックネームで照合
    for (var eDj = aRemoveRegularUsername.length; eDj--;) {
        var re = new RegExp(aRemoveRegularUsername[eDj]);
        if (re.test(username)) {
            return 3;
        }
    }
    // 完全一致ニックネームで照合 (※ニックネームが長すぎるとき、KJではコメント表示において ニックネームの後方を「…」で省略してしまうことに留意)
    if (aRemoveUsername.indexOf(username) >= 0) {
        return 2;
    }
    // ユーザーIDで照合
    if (aRemoveUserid.indexOf(userid) >= 0) {
        return 1;
    }

    // ストーカー判定(特定IDのスレッドに粘着するID)
    if (fStalkingRemove) {
        if (ownerid != null) {
            // ストーカー
            for (let stalking of StalkingList) {
                if (stalking.userids.indexOf(ownerid) >= 0 && stalking.stalkers.indexOf(userid) >= 0) {
                    // 被ストーキングIDとストーキングIDが一致したので除去する
                    el.setAttribute('data-stalker', true);
                    return 4;
                }
            }
        }
        // フェイカー(偽物)
        if (faker) {
            if (fBadNicknameRemove && badname) {
                return 8;
            }
            return 6;
        }
    }
    if (fBadNicknameRemove && badname) {
        if (fStalkingRemove && faker) {
            return 8;
        }
        return 7;
    }
    // スバム判定
    if (fSpamRemove && spam) {
        return 5;
    }

    el.setAttribute('data-remove', false);
    return 0;
}

// [関数] 強調対象か判定
function checkMark(userid, username) {
    // 正規表現ニックネームで照合
    for (var eDj = aMarkupRegularUsername.length; eDj--;) {
        var re = new RegExp(aMarkupRegularUsername[eDj]);
        if (re.test(username)) {
            return 3;
        }
    }
    // 完全一致ニックネームで照合 (※ニックネームが長すぎるとき、KJではコメント表示において ニックネームの後方を「…」で省略してしまうことに留意)
    if (aMarkupUsername.indexOf(username) >= 0) {
        return 2;
    }
    // ユーザーIDで照合
    if (aMarkupUserid.indexOf(userid) >= 0) {
        return 1;
    }

    return 0;
}

// [関数] 偽物か判定
function checkFaker(userid, username, ownerid) {
    // フェイカー(偽物)
    for (let faker of FakerList) {
        if (faker.fakers.indexOf(userid) >= 0) {
            // 被フェイカーIDとフェイカーIDが一致したので除去する
            return true;
        }
    }
    return false;
}

// [関数] スパム判定
function checkSpam(userid, category) {
    // スパムリストチェック
    for (let spam of SpamList) {
        let level = typeof spam.level !== 'undefined' ? spam.level : 3;
        if ((SpamLevel == 1 || level >= SpamLevel) && spam.userids.indexOf(userid) >= 0) {
            if (SpamLevel >= 2 && typeof spam.exception !== 'undefined') {
                // スパム評価から除外するカテゴリーを考慮
                if (spam.exception.indexOf(category) == -1) {
                    // 評価除外カテゴリーではないので、スバムIDとして除去する
                    return true;
                }
            } else {
                // スバムIDを発見したので除去する(ユーザーが設定したスパム除去レベルが1の時は、評価除外を無視して除去)
                return true;
            }
        }
    }
    return false;
}

// [関数] コメント書き込みボタンをリビルド(BAN)
function CommentPostRebuild() {
    /*
	var fPostDisabled = login.badname;
	var elC1 = document.getElementById("co_content");
	var elC2 = document.getElementById("btn_submit");
	if (elC1) {
		elC1.disabled = fPostDisabled;
	}
	if (elC2) {
		elC2.disabled = fPostDisabled;
	}
	*/
}

// [関数] スレッド書き込みボタンをリビルド(BAN)
function ThreadPostRebuild() {
    /*
	var fPostDisabled = login.badname;
	var elC1 = document.getElementById("wr_subject");
	var elC2 = document.getElementById("write_submit");
	if (elC1) {
		elC1.disabled = fPostDisabled;
	}
	if (elC2) {
		elC2.disabled = fPostDisabled;
	}
	*/
}

// [関数] ID情報のストレージ
function StorageIDInformation(info) {
    if (fUserIDStorageRefresh) {
        browser.runtime.sendMessage({
                request: "setStorageIDInformation",
                userid: info.userid,
                username: info.username,
                nation: info.nation,
                category: info.category,
                datetime: info.datetime
            },
            function(resp) {}
        );
    }
}

// [関数] ID情報の添加
function AddUserIDContentOnThread(elRecord, userid, username, message) {
    // コメントレコードのヘッダにIDを表示させる拡張 (2017.8.6.59)
    var els = elRecord.getElementsByClassName("text-muted"); // ニックネーム要素の次にIDを表示させる介入
    if (els) {
        var elId = els[0];
        elId.setAttribute("title", UserTooltipString(userid, username, message));
    }
}

// [関数] ID情報の添加
function AddUserIDContentOnComment(elRecord, userid, username, message) {
    // コメントレコードのヘッダにIDを表示させる拡張 (2017.8.6.59)
    var els = elRecord.getElementsByClassName("text-muted"); // ニックネーム要素の次にIDを表示させる介入
    if (els) {
        var elId = els[0];
        var txt = "";
        if (userid != null) {
            txt = txt + "USER ID: #" + userid;
        } else {
            txt = txt + "USER ID: *Undetectable";
        }
        txt = txt + "\n";
        txt = txt + "NICKNAME: " + username;
        txt = txt + " (" + username.length + " characters)";
        txt = txt + "\n";
        txt = txt + "ENCODE: " + username.encodeHexNumReg();
        // +"数値文字参照表記 "+username.encodeHexNumRef()+"\n"
        if (message != "") {
            txt = txt + "\n" + message;
        }
        elId.setAttribute("title", txt);
        var elChk = els[0].nextSibling;
        if (elChk && elChk.className == "kjuserid") {} else {
            var elId = document.createElement("span");
            elId.className = "kjuserid";
            elId.setAttribute("title", UserTooltipString(userid, username, message));
            elId.setAttribute("onclick", "textSelect(this)");
            if (userid != null) {
                elId.textContent = "#" + userid;
            }
            els[0].parentNode.insertBefore(elId, els[0].nextSibling);
        }
    }
}

// [イベント受信関数] ロード
function onBoardLoad() {
    // アンロードイベントを登録する
    window.addEventListener('pagehide', onBoardUnload, {
        once: true
    });

    // スクリプトを注入
    ScriptLoader(["KJAnalyze/insert-board.js"]);

    // [関数] オンロード時に処理を実施する
    // [実行] ストレージデータ獲得成功をトリガーにして施術を実施する。
    browser.storage.local.get(
        [
            "kjclub_userfiltering", "kjclub_userfiltering_nicknames", "kjclub_userfiltering_perfectly", "kjclub_userfiltering_badnickname", "kjclub_userfiltering_blacklist_stalking",
            "kjclub_userfiltering_blacklist_spam", "kjclub_userfiltering_blacklist_spam_level",
            "kjclub_currentlogin_badname",
            "kjclub_usermarking", "kjclub_usermarking_nicknames"
        ],

        (res) => {
            fRemove = fMarkup = fPerfectly = fBadNicknameRemove = fStalkingRemove = fSpamRemove = false;
            if (DFProf("kjclub_userfiltering", res.kjclub_userfiltering)) {
                fRemove = true;
                fPerfectly = DFProf("kjclub_userfiltering_perfectly", res.kjclub_userfiltering_perfectly);
                let names = DFProf("kjclub_userfiltering_nicknames", res.kjclub_userfiltering_nicknames);

                aRemoveRegularUsername = SplitListRegularNickname(names);
                aRemoveUsername = AnyUnescape(SplitListNickname(names));
                Array.prototype.push.apply(aRemoveUsername, AryDecodeNum(aRemoveUsername));
                aRemoveUserid = SplitListUserid(names);

                fBadNicknameRemove = DFProf("kjclub_userfiltering_badnickname", res.kjclub_userfiltering_badnickname);
                fStalkingRemove = DFProf("kjclub_userfiltering_blacklist_stalking", res.kjclub_userfiltering_blacklist_stalking);
                fSpamRemove = DFProf("kjclub_userfiltering_blacklist_spam", res.kjclub_userfiltering_blacklist_spam);
                SpamLevel = DFProf("kjclub_userfiltering_blacklist_spam_level", res.kjclub_userfiltering_blacklist_spam_level);
            }
            if (res.kjclub_usermarking == "1") {
                fMarkup = true;
                let names = DFProf("kjclub_usermarking_nicknames", res.kjclub_usermarking_nicknames);

                aMarkupRegularUsername = SplitListRegularNickname(names);
                aMarkupUsername = AnyUnescape(SplitListNickname(names));
                Array.prototype.push.apply(aMarkupUsername, AryDecodeNum(aMarkupUsername));
                aMarkupUserid = SplitListUserid(names);

            }

            // コンテンツの異常性を確認
            rebuildContent();

            // 不正なニックネーム
            if (res.kjclub_currentlogin_badname) {
                login.badname = true;
            } else {
                login.badname = false;
            }

            ThreadsAnalyze();
            InitializeCommentChecker();
            AccordionAnalyze();

            // スレッド書き込みフォーム・ボタンへの介入
            ThreadPostRebuild();

            try {
                // スレッド検索に機能を追加
                AdditionalSearchForm();
            } catch (e) {}

            // [ストレージ監視開始] ストレージ変化を監視し介入を動的に実行する
            browser.storage.onChanged.addListener(onBoardStorageChanged);
        }
    );
}

// ロード時に呼ばれるを登録
document.addEventListener("DOMContentLoaded", onBoardLoad);