// *************************************************************************
// [内容] メモのスクリプト。
// [詳細] メモを実現するためのメインスクリプト。
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

var memos=4; // メモ帳の数

var memo_backup=[];
for(var i=0;i<memos;i++) {
	memo_backup[i] = "";
}

// [関数] 多言語対応のためのテキストキー定義と介入の実施を行う。
function i18nRInTextsMemo() {
	// 言語介入の実行。
	i18nHtml();
}

// [関数] 初回起動時など、未記録プロファイルがあれば作成する。
function CreateNewMemoProfiles() {
	var aryProfiles=[];
	for(var i=0;i<memos;i++) {
		aryProfiles.push("memo_file"+i);
	}
	browser.storage.local.get(aryProfiles, (res) => {
		var arySet={};
		for(var i=0;i<memos;i++) {
			if( res["memo_file"+i] == undefined ) { arySet["memo_file"+i]=""; }
		}
		if(Object.keys(arySet).length>=1) {
			// 新規保存プロファイルが存在するので、ストレージにプロファイルをセット後にフォームリストアを実施する。
			browser.storage.local.set(arySet, function() {
				RestoreMemo();
			});
		}
		else {
			// 新規保存プロファイルは存在しないので直ぐにフォームリストアを実施する。
			RestoreMemo();
		}
	});
}

function RestoreMemo() {
	var aryProfiles=[];
	for(var i=0;i<memos;i++) {
		aryProfiles.push("memo_file"+i);
	}
	browser.storage.local.get(aryProfiles, (res) => {
		// console.log("KJCLUB Plus MEMO restore ok.");
		for(var i=0;i<memos;i++) {
			document.getElementById("memo"+i).value = memo_backup[i] = res["memo_file"+i] == undefined ? "" : res["memo_file"+i];
		}
	});
}

// [イベント受信関数] メモ
function onTimerCloseMemoSaved() {
	document.getElementById("help_memo_saved").style.display = "none";
}

// [イベント受信関数] メモ
function onClickMemoSave(e) {
	e.preventDefault();

	var arySet={};

	for(var i=0;i<memos;i++) {
		if( document.getElementById("memo"+i).value != memo_backup[i] ) {
			// 変化あり
			arySet["memo_file"+i]=memo_backup[i]=document.getElementById("memo"+i).value;
		}
	}

	if(Object.keys(arySet).length>=1) {
		document.getElementById("memo_save").disabled=true;
		browser.storage.local.set(arySet, function() {
			document.getElementById("help_memo_saved").style.display = "block";
			setTimeout(onTimerCloseMemoSaved,3000);
			document.getElementById("memo_save").disabled=false;
		});
	}
}

// [イベント受信関数] メモ
function onKeyupMemo(event) {
	try	{
		event=event||window.event;
		var c=event.which||a.keyCode;
		var b=typeof event.modifiers=="undefined"?event.ctrlKey:event.modifiers&Event.CONTROL_MASK;
		if(b&&c==13) {
			onClickMemoSave(event);
		}
	}
	catch(e) {
	}
}

// [イベント受信関数] オプション設定文書のロード時に一度だけコールされる
function onLoadMemo() {
	i18nRInTextsMemo();

	// [イベントリスナ登録] コンテンツロード時にonLoad()を呼ぶためのイベントリスナ登録を行う。
	document.getElementById("memo_form").addEventListener("submit", onClickMemoSave);
	document.getElementById("memo_save").addEventListener("click", onClickMemoSave);

	// [イベントリスナ登録] コンテンツロード時にonLoad()を呼ぶためのイベントリスナ登録を行う。
	for(var i=0;i<memos;i++) {
		document.getElementById("memo"+i).addEventListener("keyup", onKeyupMemo);
		document.getElementById("memo"+i).addEventListener("change", onClickMemoSave);
	}

	CreateNewMemoProfiles();
}

// [イベントリスナ登録] コンテンツロード時にonLoad()を呼ぶためのイベントリスナ登録を行う。
document.addEventListener("DOMContentLoaded", onLoadMemo);