// *************************************************************************
// [内容] バックグラウンド・スクリプト。
// [詳細] バックグラウンドサービスを提供する。
// [更新] 2023/04/19 17:58
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

// [保持] バックグラウンド・スクリプトのシャットダウン状態を占めるフラグ
// [解説] このフラグが true の時は、サービスを受け付けない。
var shutdown = false;

// [保持] 除去するユーザーのリスト(文字列)
var strUserFilteringNicknames = "";
var strUserMarkingNicknames = "";

// [関数] KJCLUB ID情報リスト
var MAXIDINFOS = 100;
var IDInfos = [];

// [配列] ログイン情報
var login = {
	userid: null,
	username: null,
	badname: false,
	ban: false
};
var beforeLogin = {
	userid: null,
	username: null,
	badname: false
};

// [配列] 最後に選択されたユーザー情報
var lastPointedUser = {
	userid: null,
	username: null
};

// [保持]
var extraview = false;

// [イベント受信関数] ストレージ変化時の動的介入
function onStorageChanged(changes, namespace) {
	if (namespace == "local") {
		if( changes.kjclub_userfiltering_nicknames ) {
			strUserFilteringNicknames = changes.kjclub_userfiltering_nicknames.newValue;
		}
		if( changes.kjclub_usermarking_nicknames ) {
			strUserMarkingNicknames = changes.kjclub_usermarking_nicknames.newValue;
		}
		if( changes.kjclub_extraview ) {
			extraview = changes.kjclub_extraview.newValue;
		}
	}
}

// [関数] ニックネームでホワイトリスト以外の文字が使われているか判定(なりすまし判定)
function isBanUserid(userid) {
	if( userid.length >= 1 ) {
		for (let faker of FakerList) {
			if (faker.fakers.indexOf(userid) >= 0) {
				// このIDがフェイカーブラックリストに登録されたIDと一致したのでBANフラグを返す
				return true;
			}
		}
		for (let warning of WarningList) {
			if (warning.userids.indexOf(userid) >= 0) {
				// このIDが警戒ブラックリストに登録されたIDと一致したのでBANフラグを返す
				return true;
			}
		}
	}
	return false;
}

// [関数] ブラウザ終了・アンインストール・無効化時等の際、このバックグラウンドスクリプトを安全に停止する処理。
// [解説] バッグラウンドサービスが利用できなくなるため、コンテンツスクリプトの正常動作も行われなくなる。
function Shutdown() {
	shutdown=true;
}

// [関数] このアドオンをアンインストールする。
// [解説] このアドオンの不正利用が疑われるとき、アンインストールを行うことで継続利用を拒絶します。
function UninstallSelf() {
	Shutdown();
	setTimeout( _UninstallSelf, 3000 );
}

function _UninstallSelf() {
	try {
		// アンインストール処理(※Firefoxのみ対応,Firefox51～)
		if( typeof browser.management == "object" ) {	// サイドバー機能がブラウザでサポートされているかを確認
			if( typeof browser.management.uninstallSelf == "function" ) {
				console.log( "KJCLUB Plus has been uninstalled." );
				browser.management.uninstallSelf( { showConfirmDialog: false, dialogMessage:"KJCLUB Plusの継続利用を拒絶します。" } );
			}
		}
	}
	catch(e) {
	}
}

// [ストレージ監視開始] ストレージ変化を監視し介入を動的に実行する
try {
	browser.storage.local.get(["kjclub_extraview", "kjclub_userfiltering_nicknames", "kjclub_usermarking_nicknames", "kjclub_cachelogin_userid", "kjclub_cachelogin_username"], (res) => {
		if ( res.kjclub_extraview != undefined ) {
			extraview = res.kjclub_extraview;
		}
		if (res.kjclub_userfiltering_nicknames != undefined) {
			strUserFilteringNicknames = res.kjclub_userfiltering_nicknames;
		}
		if (res.kjclub_usermarking_nicknames != undefined) {
			strUserMarkingNicknames = res.kjclub_usermarking_nicknames;
		}

		if (res.kjclub_cachelogin_userid != undefined) {
			beforeLogin.userid = res.kjclub_cachelogin_userid;
		}
		if (res.kjclub_cachelogin_username != undefined) {
			beforeLogin.username = res.kjclub_cachelogin_username;
		}

		browser.storage.onChanged.addListener(onStorageChanged);
	});
}
catch(e) {
}

// [関数] KJCLUB ID情報リストの追加・修正等を行う
function updateIDInfo(info) {
	for (var i = 0; i < IDInfos.length; i++) {
		if (IDInfos[i].userid == info.userid) {
			// 既に存在しているのでリストからレコードを削除し、リストの先頭に追加する
			IDInfos.splice(i, 1);
			IDInfos.unshift(info);
			// console.log("更新("+IDInfos.length+") "+info.userid+"\n"+info.username+"\n"+info.nation);
			return;
		}
	}

	// レコードは存在しないのでリストの先頭に追加する
	IDInfos.unshift(info);
	// console.log("追加("+IDInfos.length+") "+info.userid+"\n"+info.username+"\n"+info.nation);

	// レコードが多い場合は最後尾から削除する
	if (IDInfos.length > MAXIDINFOS) {
		IDInfos.splice(MAXIDINFOS);
		// console.log(MAXIDINFOS+"オーバーのため除去");
	}
}

// [イベントリスナ登録] コンテンツスクリプトからの通信を受ける
browser.runtime.onMessage.addListener(
	function(request, sender, sendResponse) {
		try {
			if( shutdown ) {
				console.log("KJCLUB Plus has shutdowned.");
				sendResponse("FAILED");
				return false;
			}
			if (request.request == "contextMenusUpdate") {
				// ユーザーIDに関するコンテキストメニュー内容の更新を要求された
				try {
					lastPointedUser.userid = request.userid;
					lastPointedUser.username = request.username;
					var visibleRemove = updateRemove(request.userid, request.username);
					var visibleMark = updateMark(request.userid, request.username);
					updateCTXMenuOptions( !visibleRemove && !visibleMark );
				}
				catch(e) {
				}
				sendResponse("OK");
				return false;
			} else
			if (request.request == "notifications") {
				// BasicNotify(request.title, request.message);
				sendResponse("OK");
				return false;
			} else
			if (request.request == "switchRemove") {
				try {
					switchRemove(request.userid, request.username);
				}
				catch(e) {
				}
				sendResponse("OK");
				return false;
			} else
			if (request.request == "switchMark") {
				try {
					switchMark(request.userid, request.username);
				}
				catch(e) {
				}
				sendResponse("OK");
				return false;
			} else
			if (request.request == "setStorageIDInformation") {
				try {
					updateIDInfo({
						userid: request.userid,
						username: request.username,
						nation: request.nation,
						category: request.category,
						datetime: request.datetime
					});
				}
				catch(e) {
				}
				sendResponse("OK");
				return false;
			} else
			if (request.request == "setLoginIDInformation") {
				try {
					login = {
						userid: request.userid,
						username: request.username,
						badname: false
					};

					// ニックネームが収得できているか?
					if (login.userid != null && login.username == null) {
						// 収得できていない場合、前回のuseridと照合して一致したら、usernameも一致していると見なす
						if (login.userid == beforeLogin.userid) {
							login.username = beforeLogin.username;
						}
					}

					if (login.userid != null) {
						// beforeLoginは login.userid が有効なときだけ更新する
						beforeLogin = {
							userid: login.userid,
							username: login.username
						};
						login.ban = isBanUserid(login.userid);
					}

					if (login.username != null) {
						login.badname = isBadNickname(login.username);
					} else {
						login.badname = false;
					}

					// 現在のログイン状態をストレージする (時刻は秒でUNIX時刻で秒記録)
					browser.storage.local.set({
						kjclub_cachelogin_userid: (beforeLogin.userid == null ? "" : beforeLogin.userid),
						kjclub_cachelogin_username: (beforeLogin.username == null ? "" : beforeLogin.username),
						kjclub_currentlogin_datetime: (new Date()).getTime(),
						kjclub_currentlogin_userid: (login.userid == null ? "" : login.userid),
						kjclub_currentlogin_username: (login.username == null ? "" : login.username),
						kjclub_currentlogin_badname: login.badname
					}, function() { 
						if( login.ban ) {
							// 不正利用ユーザーの場合は、このタイミングでこのアドオンをアンインストールする。
							UninstallSelf();
						}
						sendResponse("OK"); 
					});
				}
				catch(e) {
				}
				
				return true;
			} else
			if (request.request == "getExtraView") {
				sendResponse({
					response: "OK",
					extraview: extraview
				});
				return false;
			} else
			if (request.request == "getLoginIDInformation") {
				sendResponse({
					response: "OK",
					info: login
				});
				return false;
			} else
			if (request.request == "getIDInformations") {
				sendResponse({
					response: "OK",
					infos: IDInfos
				});
				return false;
			} else
			if (request.request == "getLastPointedUser") {
				sendResponse({
					response: "OK",
					info: lastPointedUser
				});
				return false;
			} else
			if (request.request == "uninstall") {
				try {
					UninstallSelf( request.message );
				}
				catch(e) {
				}
				sendResponse({
					response: "OK",
				});
				return false;
			}
			// 処理が行われなかった場合は FAILED を返答
			sendResponse("FAILED");
		}
		catch(e) {
		}
		return false;
	}
);