// *************************************************************************
// [内容] バックグラウンド・スクリプト／コンテキストメニュー
// [詳細] コンテキストメニュー「除去」のコントロールを実施する
// [更新] 2023/04/19 17:58
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

// [関数] ユーザーIDリムーバーメニューの実行
function clickedRemove(info, tab) {
    // browser.tabs.sendMessage(tab.id, "lastPointedUser", function(lastPointedUser) {
    if (lastPointedUser.userid != null || lastPointedUser.username != null) {
        switchRemove(lastPointedUser.userid, lastPointedUser.username);
    }
    // });
}

// [関数] 状態をセットする
function setRemove(userid, username) {
    // フィルタリストの更新
    var aSlRegularNicknames = SplitListRegularNickname(strUserFilteringNicknames);
    var aSlNicknames = SplitListNickname(strUserFilteringNicknames);
    var aSlUnescapedNicknames = AnyUnescape(SplitListNickname(strUserFilteringNicknames));
    var aSlUserids = SplitListUserid(strUserFilteringNicknames);

    // ニックネーム設定が存在しない場合は追記する
    var resultName = aSlUnescapedNicknames.indexOf(username);
    if (resultName >= 0) {} else {
        if (isBadNickname(username)) {
            // 不正なコードが含まれている場合はエンコードしてリストに格納すること
            aSlNicknames.unshift(username.encodeHexNumReg()); // このニックネームを\uエンコードしてリスト先頭に追加
        } else {
            aSlNicknames.unshift(username); // このニックネームをリスト先頭に追加
        }
        updateRemoveStorage(aSlUserids, aSlNicknames, aSlRegularNicknames); // ストレージを更新する
    }
}

// [関数] 状態を削除する
function deleteRemove(userid, username) {
    // フィルタリストの更新
    var aSlRegularNicknames = SplitListRegularNickname(strUserFilteringNicknames);
    var aSlNicknames = SplitListNickname(strUserFilteringNicknames);
    var aSlUnescapedNicknames = AnyUnescape(SplitListNickname(strUserFilteringNicknames));
    var aSlUserids = SplitListUserid(strUserFilteringNicknames);

    // ニックネーム設定が存在する場合はそれを除去する
    var resultName = aSlUnescapedNicknames.indexOf(username);
    if (resultName >= 0) {
        aSlNicknames.splice(resultName, 1); // このニックネームをリストから削除
        updateRemoveStorage(aSlUserids, aSlNicknames, aSlRegularNicknames); // ストレージを更新する
    }
}

// [関数] 状態をスイッチする
function switchRemove(userid, username) {
    deleteMark(userid, username); // この操作を実施するときは強調リストからニックネームを除去する

    var aSlNicknames = AnyUnescape(SplitListNickname(strUserFilteringNicknames));
    var resultName = aSlNicknames.indexOf(username);
    if (resultName >= 0) {
        deleteRemove(userid, username);
    } else {
        setRemove(userid, username);
    }
}

// [関数] 状態をストレージに保存する
function updateRemoveStorage(aSlUserids, aSlNicknames, aSlRegularNicknames) {
    var list = JoinListAry(aSlUserids, aSlNicknames, aSlRegularNicknames); // リスト文字列を再構築
    browser.storage.local.set({
        kjclub_userfiltering_nicknames: list,
        kjclub_userfiltering: true
    }, function() {});
}

// [右クリックしたときに開くコンテキストメニューを登録] 直接除去設定 (※Android版はcontextMenus非対応なのでtry～catchでくくる)
var cmiRemove = null;
try {
    cmiRemove = browser.contextMenus.create({
        title: i18n("optdirect_idctl_remove_contextMenus", "Remove this KJCLUB User"),
        contexts: ["all"],
        type: "checkbox",
        documentUrlPatterns: ["*://www.kjclub.com/jp/board/exc_board_*", "*://www.kjclub.com/kr/board/exc_board_*"],
        onclick: clickedRemove
    });
} catch (e) {}

// targetUrlPatterns: ["javascript:;"],

// [関数] ユーザーIDリムーバーメニューの表示更新
function updateRemove(userid, username) {
    if (!cmiRemove) {
        return false;
    }
    var mOpt = null;
    try {
        if (userid != null || username != null) {
            var aSlNicknames = AnyUnescape(SplitListNickname(strUserFilteringNicknames));
            var resultName = aSlNicknames.indexOf(username);
            if (resultName >= 0) {
                // ニックネームがリストに存在
                mOpt = {
                    visible: true,
                    checked: true
                };
            } else {
                // ニックネームはリストに存在しない
                mOpt = {
                    visible: true,
                    checked: false
                };
            }
        } else {
            mOpt = {
                visible: false,
                checked: false
            };
        }
        if (mOpt != null) {
            browser.contextMenus.update(cmiRemove, mOpt);
        }
    } catch (e) {}
    return mOpt ? mOpt.visible : false;
}