// *************************************************************************
// [内容] 多言語支援関数群。
// [詳細] 多言語表示対応を簡易に実現するための関数群。
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

// [関数] 多言語対応させる。ID=キーの要素を探し、あれば要素に囲われた文字を置換する。
function i18nRInText(key) {
	var e = document.getElementById(key);
	if (e) {
		// 要素が存在するので定義群を参照して書き換えを実施する。
		e.textContent = i18n(key, e.textContent);
	}
}

// [関数] キー(key)を元にメッセージ定義群から文字列を獲得する。獲得できない場合はdefを返す。
function i18n(key, def) {
	var n = def;
	try {
		n = browser.i18n.getMessage(key, def);
	}
	catch(e) {
	}
	return n ? n : (def ? def : key);
}

// [関数] HTMLページ内のdata-i18n属性に応じて置換する
function i18nHtml() {
	try {
		// 言語ファイルから充填
		document.querySelectorAll("[data-i18n-text]").forEach(element => {
			try {
				let n = browser.i18n.getMessage(element.getAttribute("data-i18n-text"), null);
				if (n != null) {
					element.textContent = n;
				}
			}
			catch(e) {
			}
		});
		document.querySelectorAll("[data-i18n-value]").forEach(element => {
			try {
				let n = browser.i18n.getMessage(element.getAttribute("data-i18n-value"), null);
				if (n != null) {
					element.value = n;
				}
			}
			catch(e) {
			}
		});
		// HTML内から充填
		var uiLanguage = browser.i18n.getUILanguage();
		var lang=uiLanguage.slice(0,2);
		document.querySelectorAll("[data-i18n-text-"+lang+"]").forEach(element => {
			try {
				let n = element.getAttribute("data-i18n-text-"+lang);
				if (n != null) {
					element.textContent = n;
				}
			}
			catch(e) {
			}
		});
		document.querySelectorAll("[data-i18n-value-"+lang+"]").forEach(element => {
			try {
				let n = element.getAttribute("data-i18n-value-"+lang);
				if (n != null) {
					element.value = n;
				}
			}
			catch(e) {
			}
		});
	}
	catch(e) {
	}
}