// *************************************************************************
// [内容] オプション設定スクリプト。
// [詳細] オプション設定を実現するためのメインスクリプト。
// *************************************************************************
// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

var aSideSelected = "unknown";
var aSecureSelected = "unknown";

// [プロファイル] このセクションで使用する全てのプロファイルの定義。
var aProfiles = [
	"kjclub_side",
	"kjclub_secure",

	"kjclub_login_id", "kjclub_login_pass",
	"kjclub_currentlogin_datetime", "kjclub_currentlogin_userid", "kjclub_currentlogin_username", "kjclub_currentlogin_badname",
	"kjclub_cachelogin_userid", "kjclub_cachelogin_username",

	"kjclub_automaticfiltering", "kjclub_automaticfiltering_level", "kjclub_automaticfiltering_perfectly",
	"kjclub_userfiltering", "kjclub_userfiltering_nicknames", "kjclub_userfiltering_perfectly", "kjclub_userfiltering_badnickname", "kjclub_userfiltering_blacklist_stalking",
	"kjclub_userfiltering_blacklist_spam", "kjclub_userfiltering_blacklist_spam_level",
	"kjclub_usermarking", "kjclub_usermarking_nicknames"
];

if (!AMO) {
	var aExtraProfiles = [
		"kjclub_extraview",
		"netfont7_login_id", "netfont7_login_pass"
	]; // "kjclub_extraview_sidebar_navigation",
	Array.prototype.push.apply(aProfiles, aExtraProfiles);
}

// [変化前プロファイル保持] 変化プロファイルのみ記録するため。
var Before = {};

for (idx in aProfiles) {
	var key = aProfiles[idx];
	Before[key] = "";
}

// [関数] 初回起動時など、未記録プロファイルがあれば作成する。
function CreateNewProfiles() {
	browser.storage.local.get(aProfiles, (res) => {
		var arySet = {};

		// console.log( "RESTORE!" );
		for( let key in DefaultProfiles ) {
			// console.log( "Defaults "+key+"="+DefaultProfiles[key] );
			if( res[key] == undefined ) {
				// console.log( key+" は undefined のため、デフォルト値 "+DefaultProfiles[key]+" を新規設定します。" );
				arySet[key] = DefaultProfiles[key];
			}
			else {
				if( typeof DefaultProfiles[key] == "boolean" ) {
					// 本則が true または false 要素の場合、旧プロファイルが "1" または "0" の場合に本則に矯正する
					if( typeof res[key] != "boolean" ) {
						if( res[key] == "1" ) {
							arySet[key] = true;
							// console.log( key+" は旧方式のため "+res[key]+" → "+arySet[key]+" に矯正します。" );
						}
						else
						if( res[key] == "0" ) {
							arySet[key] = false;
							// console.log( key+" は旧方式のため "+res[key]+" → "+arySet[key]+" に矯正します。" );
						}
						else {
							arySet[key] = DefaultProfiles[key];
							// console.log( key+" は不明な形式のため "+res[key]+" → "+arySet[key]+" に設定します。" );
						}
					}
				}
			}
		}

		if (Object.keys(arySet).length >= 1) {
			// 新規保存プロファイルが存在するので、ストレージにプロファイルをセット後にフォームリストアを実施する。
			browser.storage.local.set(arySet, function() {
				// console.log("KJCLUB Plus はプロファイルをストレージに書き込みました。");
				RestoreOptions();
			});
		} else {
			// 新規保存プロファイルは存在しないので直ぐにフォームリストアを実施する。
			RestoreOptions();
		}
	});
}

// [関数] ストレージの設定値からフォームへの復帰を実施する関数。
function RestoreOptions() {

	browser.storage.local.get(aProfiles, (res) => {
		// Category: kjclub_side
		FValue("kjclub_side", res.kjclub_side );
		FCheck("kjclub_side_secure", res.kjclub_secure );

		// Category: kjclub_login
		FValue("kjclub_login_id", res.kjclub_login_id );
		FValue("kjclub_login_pass", res.kjclub_login_pass );

		// Category: kjclub_currentlogin 現在もしくは最後に検出されたログイン状態
		var datetime = Before["kjclub_currentlogin_datetime"] = res.kjclub_currentlogin_datetime == undefined ? "" : res.kjclub_currentlogin_datetime;
		var strDatetime = "時刻なし";
		try {
			if (datetime != 0) {
				strDatetime = new Date(datetime);
			}
		}
		catch(e) {
		}
		document.getElementById("kjclub_currentlogin_datetime").textContent = strDatetime;
		var curLogin = {
			user: "",
			message: "状態不明",
			encode: "",
			smessage: "",
			support: false,
			userid: Before["kjclub_currentlogin_userid"] = res.kjclub_currentlogin_userid == undefined ? "" : res.kjclub_currentlogin_userid,
			username: Before["kjclub_currentlogin_username"] = res.kjclub_currentlogin_username == undefined ? "" : res.kjclub_currentlogin_username,
			badname: Before["kjclub_currentlogin_badname"] = res.kjclub_currentlogin_badname == undefined ? false : res.kjclub_currentlogin_badname
		};

		if (curLogin.username == "") {
			curLogin.user = "情報なし";
			if (curLogin.userid != "") {
				curLogin.message = "ログイン状態と推定と推定されます。";
				curLogin.user = curLogin.user + " (#" + curLogin.userid + "の情報が残存)";
			} else {
				curLogin.message = "ログアウト状態と推定されます。";
			}
		} else {
			curLogin.message = "ログイン状態と推定されます。";
			// ↓不穏なニックネームの動作チェック
			// curLogin.username=curLogin.username+String.fromCharCode(0xfefe);
			curLogin.user = curLogin.username + "#" + curLogin.userid;
			if (Before["kjclub_currentlogin_badname"]) {
				curLogin.message = "現在ログイン状態ですが、ニックネームにポリシー上許容できない不穏な文字コードが混入しています。継続してKJCLUBを利用する場合は、" + i18n("KJPlusExtensionName", "KJCLUB Plus") + "アドオンを削除してください。";
				curLogin.smessage = "なお、誤検出の場合は修正に応じますので、お手数ですが以下のコードを提出してください。(※文字によっては残念ながら修正に応じられない場合もあります)";
				curLogin.support = true;
				curLogin.encode = curLogin.username.encodeHexNumReg() + "#" + curLogin.userid;
			}
		}

		IText("kjclub_currentlogin_user", curLogin.user);
		IText("kjclub_currentlogin_message", curLogin.message);
		IText("kjclub_currentlogin_encode", curLogin.encode);
		IText("kjclub_currentlogin_support", curLogin.smessage);

		for (const id of["kjclub_currentlogin_encode", "kjclub_currentlogin_support", "kjclub_currentlogin_supporturl"]) {
			document.getElementById(id).style.display = curLogin.support ? "block" : "none";
		}

		FCheck("kjclub_automaticfiltering", res.kjclub_automaticfiltering );
		FValue("kjclub_automaticfiltering_level", res.kjclub_automaticfiltering_level == undefined ? 50 : res.kjclub_automaticfiltering_level);
		FCheck("kjclub_automaticfiltering_perfectly", res.kjclub_automaticfiltering_perfectly );

		FCheck("kjclub_userfiltering", res.kjclub_userfiltering );
		FValue("kjclub_userfiltering_nicknames", trimNicknames(res.kjclub_userfiltering_nicknames == undefined ? "" : res.kjclub_userfiltering_nicknames));
		FCheck("kjclub_userfiltering_perfectly", res.kjclub_userfiltering_perfectly );
		FCheck("kjclub_userfiltering_badnickname", res.kjclub_userfiltering_badnickname );
		FCheck("kjclub_userfiltering_blacklist_stalking", res.kjclub_userfiltering_blacklist_stalking );

		FCheck("kjclub_userfiltering_blacklist_spam", res.kjclub_userfiltering_blacklist_spam );
		FValue("kjclub_userfiltering_blacklist_spam_level", res.kjclub_userfiltering_blacklist_spam_level );

		FCheck("kjclub_usermarking", res.kjclub_usermarking );
		FValue("kjclub_usermarking_nicknames", trimNicknames(res.kjclub_usermarking_nicknames == undefined ? "" : res.kjclub_usermarking_nicknames));

		// [拡張] 署名のみ版(AMO審査を行わない)用の機能
		if (!AMO) {
			FCheck("kjclub_extraview", res.kjclub_extraview );
			FValue("netfont7_login_id", res.netfont7_login_id );
			FValue("netfont7_login_pass", res.netfont7_login_pass );
		}

		// Update masks
		maskOptions();

		// KJCLUB ユーザーIDリスティングの実施 (※AMO版では処理は行われない)
		KJIDListUpdate();
	});
}

// [関数] フォームのチェック
function FCheck(name, chk) {
	if( chk == undefined || ( typeof chk != "boolean" && (chk == "1" || chk == "0") ) ) {
		// console.log("FCheck : "+name+"の値が"+chk+"のため、デフォルト値"+DefaultProfiles[name]+"に矯正します。" )
		chk=DefaultProfiles[name];
	}
	document.forms["options"].elements[name].checked = Before[name] = chk;
}

// [関数] フォームの値セット
function FValue(name, val) {
	if( val == undefined ) {
		// console.log("FValue : "+name+"の値が"+val+"のため、デフォルト値"+DefaultProfiles[name]+"に矯正します。" )
		val=DefaultProfiles[name];
	}
	document.forms["options"].elements[name].value = Before[name] = val;
}

// [関数] テキストのセット
function IText(id, val) {
	document.getElementById(id).textContent = val;
}

// [関数] フォームのチェック状態でフィールドマスクを実施する関数。
function maskOptions() {
	var frm = document.forms["options"];

	// Category: kjclub_side
	i18nKJLinks(frm.kjclub_side.value, frm.kjclub_side_secure.checked);

	// Category: kjclub_optionlinks
	// frm.kjclub_optionlinks.disabled=false;
	// var sFrmOptionsLinksDisplay=frm.kjclub_optionlinks.checked ? "block" : "none";
	// document.getElementById("frm_kjclub_optionlinks").style.display=sFrmOptionsLinksDisplay;

	// Category: kjclub_automaticfiltering
	frm.kjclub_automaticfiltering.disabled = false;
	var kjclub_automaticfiltering_disabled = !frm.kjclub_automaticfiltering.checked;
	frm.kjclub_automaticfiltering_level.disabled = kjclub_automaticfiltering_disabled;
	frm.kjclub_automaticfiltering_perfectly.disabled = kjclub_automaticfiltering_disabled;

	// Category: kjclub_userfiltering
	frm.kjclub_userfiltering.disabled = false;
	var kjclub_userfiltering_disabled = !frm.kjclub_userfiltering.checked;
	frm.kjclub_userfiltering_nicknames.disabled = kjclub_userfiltering_disabled;
	frm.kjclub_userfiltering_perfectly.disabled = kjclub_userfiltering_disabled;
	frm.kjclub_userfiltering_badnickname.disabled = kjclub_userfiltering_disabled;
	frm.kjclub_userfiltering_blacklist_stalking.disabled = kjclub_userfiltering_disabled;

	frm.kjclub_userfiltering_blacklist_spam.disabled = kjclub_userfiltering_disabled;
	var kjclub_userfiltering_blacklist_spam_disabled = kjclub_userfiltering_disabled || !frm.kjclub_userfiltering_blacklist_spam.checked;
	frm.kjclub_userfiltering_blacklist_spam_level.disabled = kjclub_userfiltering_blacklist_spam_disabled;

	// Category: kjclub_usermarking
	frm.kjclub_usermarking.disabled = false;
	var kjclub_usermarking_disabled = !frm.kjclub_usermarking.checked;
	frm.kjclub_usermarking_nicknames.disabled = kjclub_usermarking_disabled;

	if (!AMO) {
		frm.kjclub_extraview.disabled = false;
		var kjclub_extraview_disabled = !frm.kjclub_extraview.checked;
		// frm.kjclub_extraview_sidebar_navigation.disabled=kjclub_extraview_disabled;
	}
}

// [関数] 前後をトリムした上で、入力を便利にするために最後に空白を追加する関数。
function trimNicknames(nicnms) {
	var txt = nicnms.trim();
	if (txt.length >= 1) {
		txt = txt + " ";
	}
	return txt;
}

// [関数] リンクの言語とリンク先を変更する関数。
function i18nKJLinks(side, secure) {
	if (aSideSelected == side && aSecureSelected == secure) {
		return;
	}

	aSideSelected = side;
	aSecureSelected = secure;

	var theme = ["exc_board_1", "exc_board_2", "exc_board_3", "exc_board_4", "exc_board_5", "exc_board_6", "exc_board_7", "exc_board_8", "exc_board_9", "exc_board_68", "exc_board_10", "exc_board_11", "exc_board_12", "exc_board_50", "exc_board_51"];
	var photo = ["exc_board_13", "exc_board_14", "exc_board_24", "exc_board_15", "exc_board_16", "exc_board_17", "exc_board_18", "exc_board_19", "exc_board_20", "exc_board_21", "exc_board_22", "exc_board_23", "exc_board_52", "exc_board_53"];

	// http? https?
	// var kjclubScheme = secure ? "https" : "http";
	var kjclubScheme = "https"; // 2017/4/20 KJCLUB disabled https access.

	// Main links
	var kjclubSide = (side == "kr") ? "kr" : "jp";
	document.getElementById("opt_kjclub").setAttribute("href", kjclubScheme + "://www.kjclub.com/" + kjclubSide + "/");
	document.getElementById("opt_penpal").setAttribute("href", kjclubScheme + "://www.kjclub.com/" + kjclubSide + "/friends");
	document.getElementById("opt_club").setAttribute("href", kjclubScheme + "://www.kjclub.com/" + kjclubSide + "/cafe");
	document.getElementById("opt_trans").setAttribute("href", kjclubScheme + "://www.kjclub.com/" + kjclubSide + "/openchat");
	document.getElementById("opt_exchange").setAttribute("href", kjclubScheme + "://www.kjclub.com/" + kjclubSide + "/board");

	// テーマ掲示板
	for (var key of theme) {
		var e = document.getElementById(key);
		if (e) {
			// 要素が存在する
			var href = "#";
			if (side == "jp") {
				href = kjclubScheme + "://www.kjclub.com/jp/board/" + key;
			} else if (side == "kr") {
				href = kjclubScheme + "://www.kjclub.com/kr/board/" + key;
			}
			e.setAttribute("href", href);
		}
	}

	// フォト掲示板
	for (var key of photo) {
		var e = document.getElementById(key);
		if (e) {
			// 要素が存在する
			var href = "#";
			if (side == "jp") {
				href = kjclubScheme + "://www.kjclub.com/jp/board/" + key;
			} else if (side == "kr") {
				href = kjclubScheme + "://www.kjclub.com/kr/board/" + key;
			}
			e.setAttribute("href", href);
		}
	}
}

// [関数] 多言語対応のためのテキストキー定義と介入の実施を行う。
function i18nRInTextsOptions() {
	document.title = i18n("sidebar_tab_options", "KJCLUB Plus+ Settings");
	// 言語介入の実行。
	i18nHtml();
}

// [イベント受信関数] フォームからストレージへ設定値の保存を実施する。
function saveChanges() {
	var frm = document.forms["options"];

	var arySet = {};

	if (frm.kjclub_side.value != Before["kjclub_side"]) {
		arySet["kjclub_side"] = frm.kjclub_side.value;
		Before["kjclub_side"] = frm.kjclub_side.value;
	}
	if (frm.kjclub_side_secure.checked != Before["kjclub_secure"]) {
		arySet["kjclub_secure"] = frm.kjclub_side_secure.checked;
		Before["kjclub_secure"] = frm.kjclub_side_secure.checked;
	}
	if (frm.kjclub_login_id.value != Before["kjclub_login_id"]) {
		arySet["kjclub_login_id"] = frm.kjclub_login_id.value.trim();
		Before["kjclub_login_id"] = frm.kjclub_login_id.value;
	}
	if (frm.kjclub_login_pass.value != Before["kjclub_login_pass"]) {
		arySet["kjclub_login_pass"] = frm.kjclub_login_pass.value.trim();
		Before["kjclub_login_pass"] = frm.kjclub_login_pass.value;
	}
	if (frm.kjclub_automaticfiltering.checked != Before["kjclub_automaticfiltering"]) {
		arySet["kjclub_automaticfiltering"] = frm.kjclub_automaticfiltering.checked;
		Before["kjclub_automaticfiltering"] = frm.kjclub_automaticfiltering.checked;
	}
	if (frm.kjclub_automaticfiltering_level.value != Before["kjclub_automaticfiltering_level"]) {
		arySet["kjclub_automaticfiltering_level"] = frm.kjclub_automaticfiltering_level.value;
		Before["kjclub_automaticfiltering_level"] = frm.kjclub_automaticfiltering_level.value;
	}
	if (frm.kjclub_automaticfiltering_perfectly.checked != Before["kjclub_automaticfiltering_perfectly"]) {
		arySet["kjclub_automaticfiltering_perfectly"] = frm.kjclub_automaticfiltering_perfectly.checked;
		Before["kjclub_automaticfiltering_perfectly"] = frm.kjclub_automaticfiltering_perfectly.checked;
	}
	if (frm.kjclub_userfiltering.checked != Before["kjclub_userfiltering"]) {
		arySet["kjclub_userfiltering"] = frm.kjclub_userfiltering.checked;
		Before["kjclub_userfiltering"] = frm.kjclub_userfiltering.checked;
	}
	if (frm.kjclub_userfiltering_nicknames.value != Before["kjclub_userfiltering_nicknames"]) {
		arySet["kjclub_userfiltering_nicknames"] = frm.kjclub_userfiltering_nicknames.value.trim();
		Before["kjclub_userfiltering_nicknames"] = frm.kjclub_userfiltering_nicknames.value;
	}
	if (frm.kjclub_userfiltering_perfectly.checked != Before["kjclub_userfiltering_perfectly"]) {
		arySet["kjclub_userfiltering_perfectly"] = frm.kjclub_userfiltering_perfectly.checked;
		Before["kjclub_userfiltering_perfectly"] = frm.kjclub_userfiltering_perfectly.checked;
	}
	if (frm.kjclub_userfiltering_badnickname.checked != Before["kjclub_userfiltering_badnickname"]) {
		arySet["kjclub_userfiltering_badnickname"] = frm.kjclub_userfiltering_badnickname.checked;
		Before["kjclub_userfiltering_badnickname"] = frm.kjclub_userfiltering_badnickname.checked;
	}
	if (frm.kjclub_userfiltering_blacklist_stalking.checked != Before["kjclub_userfiltering_blacklist_stalking"]) {
		arySet["kjclub_userfiltering_blacklist_stalking"] = frm.kjclub_userfiltering_blacklist_stalking.checked;
		Before["kjclub_userfiltering_blacklist_stalking"] = frm.kjclub_userfiltering_blacklist_stalking.checked;
	}

	if (frm.kjclub_userfiltering_blacklist_spam.checked != Before["kjclub_userfiltering_blacklist_spam"]) {
		arySet["kjclub_userfiltering_blacklist_spam"] = frm.kjclub_userfiltering_blacklist_spam.checked;
		Before["kjclub_userfiltering_blacklist_spam"] = frm.kjclub_userfiltering_blacklist_spam.checked;
	}
	if (frm.kjclub_userfiltering_blacklist_spam_level.value != Before["kjclub_userfiltering_blacklist_spam_level"]) {
		arySet["kjclub_userfiltering_blacklist_spam_level"] = frm.kjclub_userfiltering_blacklist_spam_level.value;
		Before["kjclub_userfiltering_blacklist_spam_level"] = frm.kjclub_userfiltering_blacklist_spam_level.value;
	}

	if (frm.kjclub_usermarking.checked != Before["kjclub_usermarking"]) {
		arySet["kjclub_usermarking"] = frm.kjclub_usermarking.checked;
		Before["kjclub_usermarking"] = frm.kjclub_usermarking.checked;
	}
	if (frm.kjclub_usermarking_nicknames.value != Before["kjclub_usermarking_nicknames"]) {
		arySet["kjclub_usermarking_nicknames"] = frm.kjclub_usermarking_nicknames.value.trim();
		Before["kjclub_usermarking_nicknames"] = frm.kjclub_usermarking_nicknames.value;
	}

	// [拡張] 署名のみ版(AMO審査を行わない)用の機能
	if (!AMO) {
		if (frm.netfont7_login_id.value != Before["netfont7_login_id"]) {
			arySet["netfont7_login_id"] = frm.netfont7_login_id.value.trim();
			Before["netfont7_login_id"] = frm.netfont7_login_id.value;
		}
		if (frm.netfont7_login_pass.value != Before["netfont7_login_pass"]) {
			arySet["netfont7_login_pass"] = frm.netfont7_login_pass.value.trim();
			Before["netfont7_login_pass"] = frm.netfont7_login_pass.value;
		}
		if (frm.kjclub_extraview.checked != Before["kjclub_extraview"]) {
			arySet["kjclub_extraview"] = frm.kjclub_extraview.checked;
			Before["kjclub_extraview"] = frm.kjclub_extraview.checked;
		}
		// if( frm.kjclub_extraview_sidebar_navigation.checked != Before["kjclub_extraview_sidebar_navigation"] ) { arySet["kjclub_extraview_sidebar_navigation"]=frm.kjclub_extraview_sidebar_navigation.checked ? "1" : "0"; Before["kjclub_extraview_sidebar_navigation"]=frm.kjclub_extraview_sidebar_navigation.checked; }

	}

	if (Object.keys(arySet).length >= 1) {
		// 設定の変更がありプロファイルのセットが必要なので実行する。
		maskOptions(); // 保存前にマスク状態を変更する。
		// セット実行
		browser.storage.local.set(arySet, function() {
			// console.log("KJCLUB Plus storage updated.");
		});
	}
	return true;
}

// [イベント受信関数] オプション設定文書のロード時に一度だけコールされる
function onLoad() {
	i18nRInTextsOptions();

	var frm = document.forms["options"];

	// プロファイルがストレージに存在しない場合は、方式が古い場合は、再構成する
	CreateNewProfiles();

	var OptForms = [
		"kjclub_side_secure",
		"kjclub_login_id", "kjclub_login_pass",
		"kjclub_automaticfiltering", "kjclub_automaticfiltering_level", "kjclub_automaticfiltering_perfectly",
		"kjclub_userfiltering", "kjclub_userfiltering_nicknames", "kjclub_userfiltering_perfectly", "kjclub_userfiltering_badnickname", "kjclub_userfiltering_blacklist_stalking", "kjclub_userfiltering_blacklist_spam", "kjclub_userfiltering_blacklist_spam_level",
		"kjclub_usermarking", "kjclub_usermarking_nicknames"
	];

	if (!AMO) {
		Array.prototype.push.apply(OptForms, ["kjclub_extraview", "netfont7_login_id", "netfont7_login_pass"]);
	}

	for (const name of OptForms) {
		frm.elements[name].addEventListener("change", saveChanges);
	}

	document.getElementById("kjclub_side_jp").addEventListener("change", saveChanges);
	document.getElementById("kjclub_side_kr").addEventListener("change", saveChanges);
	// document.getElementById("kjclub_side_px").addEventListener("change", saveChanges);

	// [イベントリスナ登録] submitイベント
	document.querySelector("form").addEventListener("submit", function(e) {
		e.preventDefault();
		saveChanges();
	});
	// document.addEventListener('beforeunload', function(e){alert("rwar");document.getElementById("save").focus();saveChanges();});

	// [ストレージ監視開始] ストレージ変化(セキュアとアクセスサイド)を監視し、変化したら再構築させる。
	browser.storage.onChanged.addListener(onStorageChanged);

	// [イベントリスナ登録] クリックヘルプ
	document.getElementById("helpbtn_user_filtering_nicknames").addEventListener("click", function(e) {
		document.getElementById("help_user_filtering_nicknames").style.display = document.getElementById("help_user_filtering_nicknames").style.display == "block" ? "none" : "block";
	});
	document.getElementById("helpbtn_user_marking_nicknames").addEventListener("click", function(e) {
		document.getElementById("help_user_marking_nicknames").style.display = document.getElementById("help_user_marking_nicknames").style.display == "block" ? "none" : "block";
	});

	// KJCLUB ユーザーIDリスティングを手動実行する (※AMO版では処理は行われない)
	document.getElementById("kjclub_idlist_refresh").addEventListener("click", function(e) {
		KJIDListUpdate();
	});

	// KJCLUB ユーザーIDリスティングをNETFONT7.NETへ送信する (※ログインが必要)
	document.getElementById("kjclub_idlist_post").addEventListener("click", function(e) {
		KJIDListPost();
	});

}

// [イベント受信関数] ストレージ変化時の動的介入
function onStorageChanged(changes, namespace) {
	var fOptionsRebuild = false;

	if (changes.kjclub_userfiltering || changes.kjclub_userfiltering_nicknames || changes.kjclub_usermarking || changes.kjclub_usermarking_nicknames) {
		fOptionsRebuild = true;
	}

	if (changes.kjclub_currentlogin_datetime || changes.kjclub_currentlogin_userid || changes.kjclub_currentlogin_username) {
		fOptionsRebuild = true;
	}

	if (fOptionsRebuild) {
		// console.log("KJCLUB Plus storage changed.");
		RestoreOptions();
	}
}

// [イベントリスナ登録] コンテンツロード時にonLoad()を呼ぶためのイベントリスナ登録を行う。
document.addEventListener("DOMContentLoaded", onLoad);