// *************************************************************************
// [内容] バックグラウンド・スクリプト。
// [詳細] ページアクション実現のためにタブの切り替えを監視する。
// *************************************************************************

// [互換] browserの名前空間がなけれぱchromeの名前空間を使用する。
var browser = browser || chrome;

// [関数] 多言語対応のためのテキストキー定義と介入の実施を行う。
function i18nRInTextsMenus() {
	i18nHtml();
}

// [イベント受信関数] ページアクションアイコンクリック時にオプション設定タブを開く関数。
function onClickedSettings(e) {
	CreateUpdateTab("../options_ui/options.html");
}

// [イベント受信関数] サイドバーを開くメニューがクリックされた
function onClickedSidebarOpen(e) {
	document.getElementById("KJPlusSidebar").style.display="none";
	if( typeof browser.sidebarAction == "object" ) {	// サイドバー機能がブラウザでサポートされているかを確認
		if( typeof browser.sidebarAction.open == "function" ) {	// openがサポートされているか確認 (Firefoxは57以降対応)
			browser.sidebarAction.open();
			window.close();
		}
	}
}

var userid=null;
var username=null;

function popupShowing() {

	// サイドバー開閉
	document.getElementById("KJPlusSidebar").style.display="none";
	
	if( typeof browser.sidebarAction == "object" ) {	// サイドバー機能がブラウザでサポートされているかを確認
		if( typeof browser.sidebarAction.isOpen == "function" ) {	// isOpenがサポートされているか確認 (Firefoxは59以降対応)
			browser.sidebarAction.isOpen({}).then(result => {
				if( !result ) {
					document.getElementById("KJPlusSidebar").style.display="block";
				}
			});
		}
	}
	
	// IDコントロール
	userid=null;
	username=null;

	browser.runtime.sendMessage({
			request: "getLastPointedUser"
		},
		function(resp) {
			if( resp.response == "OK" ) {
				var lastPointedUser = resp.info;
				if (lastPointedUser) {
					if( lastPointedUser.userid==null ) {
						document.getElementById("KJPlusIDCtrl").style.display="none";
					}
					else {
						userid=lastPointedUser.userid;
						username=lastPointedUser.username;
						refreshPopup();
					}
				}
				else {
					document.getElementById("KJPlusIDCtrl").style.display="none";
				}
			}
		}
	);
}

// [関数] ポップアップの内容をリフレッシュする
function refreshPopup() {
	document.getElementById("KJPlusIDCtrl").style.display="block";
	document.getElementById("idctrl_username").textContent=username;
	document.getElementById("idctrl_userid").textContent="#"+userid;

	// document.getElementById("idctrl_username_numref").textContent=username.encodeHexNumRef();
	document.getElementById("idctrl_username_numreg").textContent=username.encodeHexNumReg();

	browser.storage.local.get(["kjclub_userfiltering_nicknames","kjclub_usermarking_nicknames"], (res) => {

		var strUserFilteringNicknames = DFProf( "kjclub_userfiltering_nicknames", res.kjclub_userfiltering_nicknames );
		var strUserMarkingNicknames = DFProf( "kjclub_usermarking_nicknames", res.kjclub_usermarking_nicknames );

		// チェック状態を設定
		var aSlFilteringUserids = SplitListUserid( strUserFilteringNicknames );
		var aSlMarkingUserids = SplitListUserid( strUserMarkingNicknames );

		if( aSlFilteringUserids.indexOf(userid) >=0 ) {
			document.getElementById("idctrl_chk_filtering").checked=true;
		}
		else {
			document.getElementById("idctrl_chk_filtering").checked=false;
		}
		if( aSlMarkingUserids.indexOf(userid) >=0 ) {
			document.getElementById("idctrl_chk_marking").checked=true;
		}
		else {
			document.getElementById("idctrl_chk_marking").checked=false;
		}
	});
}

function onClickRemove(tab) {

	if( userid!=null ) {
		browser.runtime.sendMessage({
				request: "switchRemove",
				userid: userid,
				username: username
			},
			function(resp) {
			}
		);
	}

	document.getElementById("KJPlusIDCtrl").style.display="none"; // Android版Firefox対策
	hidePageActionPopup();
}

function onClickMark(tab) {

	if( userid!=null ) {
		browser.runtime.sendMessage({
				request: "switchMark",
				userid: userid,
				username: username
			},
			function(resp) {
			}
		);
	}

	document.getElementById("KJPlusIDCtrl").style.display="none"; // Android版Firefox対策
	hidePageActionPopup();
}

function hidePageActionPopup() {
	try	{
		window.close();
	}
	catch(e) {
	}
}

// [イベント受信関数] メニュー文書ロード完了時にコールされる。
function onLoad(event) {
	// 多言語対応
	i18nRInTextsMenus();
	// [イベントリスナ登録]
	popupShowing();
	document.getElementById("menu_settings").addEventListener("click", onClickedSettings);
	document.getElementById("KJPlusSidebar").addEventListener("click", onClickedSidebarOpen);
	document.getElementById("idctrl_chk_filtering").addEventListener("click", onClickRemove);
	document.getElementById("idctrl_chk_marking").addEventListener("click", onClickMark);
}

// [イベントリスナ登録] サイドバーパネルのロード完了イベントを受けて介入を実施する。
document.addEventListener("DOMContentLoaded", onLoad);
